//    abc_plugin.js: Find everything which looks like abc and convert it

//    Copyright (C) 2010 Gregory Dyke (gregdyke at gmail dot com)
//
//    This program is free software: you can redistribute it and/or modify
//    it under the terms of the GNU General Public License as published by
//    the Free Software Foundation, either version 3 of the License, or
//    (at your option) any later version.
//
//    This program is distributed in the hope that it will be useful,
//    but WITHOUT ANY WARRANTY; without even the implied warranty of
//    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//    GNU General Public License for more details.
//
//    You should have received a copy of the GNU General Public License
//    along with this program.  If not, see <http://www.gnu.org/licenses/>.

//    requires: abcjs, raphael, jquery

//		modified for PmWiki AbcTunebook recipe by Hans Bracker
//		version 2010-05-24

function ABCPlugin() {
  this.show_midi = true;
  this.hide_abc = false;
  this.render_before = false;
  this.midi_options = {};
  this.parse_options = {};
  this.render_options = {};
  this.render_classname = "abcrendered";
  this.text_classname = "abctext";
  this.auto_render_threshold = 20;
  this.show_text = "";
  this.hide_text = "";
}
var abc_plugin = new ABCPlugin();

$(document).ready(start_abc);
$(document).ready(toggleAbc);

function start_abc() {
 //toggleAbc();
  abc_plugin.start();
 
   
}

ABCPlugin.prototype.start = function() {
  this.errors="";
  var elems = this.getABCContainingElements($("body"));
  var self = this;
  var divs = elems.map(function(i,elem){
      return self.convertToDivs(elem,i);
    });
  //toggleAbc();
  this.auto_render = (divs.size()<=this.auto_render_threshold);
  divs.each(function(i,elem){
      self.render(elem,i,$(elem).data("abctext"));
    });
};

// returns a jquery set of the descendants (including self) of elem which have a text node which matches "X:"
ABCPlugin.prototype.getABCContainingElements = function(elem) {
  var results = $();
  var includeself = false;
  var self = this;
  // TODO maybe look to see whether it's even worth it by using textContent ?
  $(elem).contents().each(function() { 
      if (this.nodeType == 3 && !includeself) {
	if (this.nodeValue.match(/^\s*X:/m)) {
	  results = results.add($(elem));
	  includeself = true;
	}
      } else if (this.nodeType==1) {
	results = results.add(self.getABCContainingElements(this));
      }
    });
  return results;
};

// in this element there are one or more pieces of abc 
// (and it is not in a subelem)
// for each abc piece, we surround it with a div, store the abctext in the 
// div's data("abctext") and return an array 
ABCPlugin.prototype.convertToDivs = function (elem,i) {
  var self = this;
  var contents = $(elem).contents();
  var abctext = "";
  var abcdiv = null;
  var inabc = false;
  var brcount = 0;
  var results = $();
  var idx = i+1;
  contents.each(function(i,node){
      if (node.nodeType==3 && !node.nodeValue.match(/^\s*$/)) {
	brcount=0;
	var text = node.nodeValue;
	if (text.match(/^\s*X:/m)) {
	  inabc=true;
	  abctext="";
	  var abctoggle = " <a href='#' id='toggleLink"+idx+"' class='toggleLink noprint'>show abc</a> ";
	  $(node).before(abctoggle);
	  //$('#abctop'+idx).append(abctoggle);
	  abcdiv=$("<pre id='"+self.text_classname+idx+"' class='"+self.text_classname+" toggleAbc'></pre>");
	  $(node).before(abcdiv);
	  if (self.hide_abc) {
	    abcdiv.hide();
	  } 
	}
	if (inabc) {
	  abctext += text.replace(/\n$/,"").replace(/^\n/,"");
	  abcdiv.append($(node));
	} 
      } else if (inabc && $(node).is("br") && brcount==0) {
	abctext += "\n";
	abcdiv.append($(node));
	brcount++;
      } else if (inabc) { // second br or whitespace textnode
	inabc = false;
	brcount=0;
	abcdiv.data("abctext",abctext);
	results = results.add(abcdiv);
      }
    });
  if (inabc) {
    abcdiv.data("abctext",abctext);
    results = results.add(abcdiv);
  }
  return results.get();
}


ABCPlugin.prototype.render = function (contextnode, i, abcstring) {
  var idx = i+1;
  var abcdiv = $("<div id='"+this.render_classname+idx+"' class='"+this.render_classname+"'></div>");
  if (this.render_before) {
    $(contextnode).before(abcdiv);
  } else {
    $(contextnode).after(abcdiv);
  }
  var self = this;
  try {
    var tunebook = new AbcTuneBook(abcstring);
    var abcParser = new AbcParse();
    abcParser.parse(tunebook.tunes[0].abc);
    var tune = abcParser.getTune();

    var doPrint = function() {
			try {
			  var paper = Raphael(abcdiv.get(0), 800, 400);
			  var printer = new ABCPrinter(paper,self.render_options);
			  printer.printABC(tune);
			} catch (ex) { // f*** internet explorer doesn't like innerHTML in weird situations
			  // can't remember why we don't do this in the general case, but there was a good reason
			  abcdiv.remove();
			  abcdiv = $("<div id='"+this.render_classname+idx+"' class='"+self.render_classname+"'></div>");
			  paper = Raphael(abcdiv.get(0), 800, 400);
			  printer = new ABCPrinter(paper);
			  printer.printABC(tune);
			  if (self.render_before) {
			    $(contextnode).before(abcdiv);
			  } else {
			    $(contextnode).after(abcdiv);
			  }
			}
			if (ABCMidiWriter && self.show_midi) {
			  midiwriter = new ABCMidiWriter(abcdiv.get(0),self.midi_options);
			  midiwriter.writeABC(tune);
			}
		};
		
		
    var showtext = "<a class='abcshow noprint' href='#'>"+this.show_text+(tune.metaText.title||"untitled")+"</a>";
    
    if (this.auto_render) {
      doPrint();
    //  toggleAbc();
    } else {
      var showspan = $(showtext);
      showspan.click(function(){
			  doPrint();
			  // toggleAbc();
			  showspan.hide();
			  return false;
			});
      abcdiv.before(showspan);
    }
		//abcdiv.prepend(abctoggle);
    } catch (e) {
    this.errors+=e;
   }
}

// toggle abctext
//ABCPlugin.prototype.toggleAbc = function () {
function toggleAbc() {
	var showText='show abc';
	var hideText='hide abc';
	var is_visible = false;
	//$('.toggleAbc').prev().append(' <a href="#" class="toggleLink">'+showText+'</a>');
	$('.toggleAbc').hide();
	$('a.toggleLink').click(function() {
		is_visible = !is_visible;
		$(this).html( (!is_visible) ? showText : hideText);
		$(this).next('.toggleAbc').toggle();
		//$('#sidebar').toggle();
		return false;
	});
};

//submit speedchange form on radio click
$(document).ready(function() {
	$('input.speed').click(function() {
		jQuery('form').submit();
	});
});

